/*
Problem: database
Contest: UOI 2012
Solution: Straightforward O((N + M) * K)
Author: Andrii Korotkov
*/

#include <cstdio>
#include <vector>
#include <algorithm>
#include <cstring>

using namespace std;

#define N 50005

const int RIGHTS_FROM_ADMIN = 0;
const int NO_RIGHTS_FROM_ADMIN = 1;
const int RIGHTS_TO_SUBORDINATES = 2;
const int RIGHTS_TO_SUBORDINATES_RECURSIVELY = 3;
const int NO_RIGHTS_TO_SUBORDINATES = 4;

int n,m,k;
vector<int> a[N];
bool used[N],give[N];
int rights[N][2];
int x,y,t;
int ans[N];

void give_rights_recursively(int v, bool r)
{
	used[v] = 1;
	for(int i = 0; i < a[v].size(); ++i)
	{
		if(r && (!give[v] || !ans[v]))
			rights[a[v][i]][1]++;
		if(!used[a[v][i]])
			give_rights_recursively(a[v][i], r);
	}
	ans[v] |= r;
	give[v] = 1;
}

void check_rights(int v)
{	
	int r = rights[v][0] + rights[v][1];
	if(ans[v] != (r > 0))
	{
		int d = int(r > 0) - ans[v]; // 1 when getting rights, -1 when loosing rights
		ans[v] = (r > 0);
		if(give[v])
		{
			for(int i = 0; i < a[v].size(); ++i)
			{
				rights[a[v][i]][1] += d;
				check_rights(a[v][i]);
			}
		}
	}
}

void give_rights_to_subordinates(int v)
{
	if(give[v]) return;
	give[v] = 1;
	if(!ans[v]) return;
	for(int i = 0; i < a[v].size(); ++i)
		rights[a[v][i]][1]++;
	for(int i = 0; i < a[v].size(); ++i)
		check_rights(a[v][i]);
}

void deny_rights_to_subordinates(int v)
{
	if(!give[v]) return;
	give[v] = 0;
	if(!ans[v]) return;
	for(int i = 0; i < a[v].size(); ++i)
		rights[a[v][i]][1]--;
	for(int i = 0; i < a[v].size(); ++i)
		check_rights(a[v][i]);
}

int main()
{
	freopen("database.dat","r",stdin);
	freopen("database.sol","w",stdout);
	scanf("%d%d",&n,&m);
	for(int i = 0; i < m; ++i)
	{
		scanf("%d%d",&x,&y);
		--x, --y;
		a[x].push_back(y);
	}
	scanf("%d",&k);
	for(int i = 0; i < k; ++i)
	{
		scanf("%d%d",&t,&x);
		--t, --x;
		memset(used,0,sizeof(used));
		switch(t) {
		case RIGHTS_FROM_ADMIN:
			rights[x][0] = 1;
			check_rights(x);
			break;
		case NO_RIGHTS_FROM_ADMIN:
			rights[x][0] = 0;
			check_rights(x);
			break;
		case RIGHTS_TO_SUBORDINATES:
			give_rights_to_subordinates(x);
			break;
		case RIGHTS_TO_SUBORDINATES_RECURSIVELY:
			give_rights_recursively(x, ans[x]);
			break;
		case NO_RIGHTS_TO_SUBORDINATES:
			deny_rights_to_subordinates(x);
			break;
		}
	}
	for(int i = 0; i < n; ++i)
	{
		printf("%d%c", ans[i], i < n-1 ? ' ' : '\n');
	}
	return 0;
}